##  <#GAPDoc Label="Saturate:example">
##  <Example><![CDATA[
##  gap> ZZ := HomalgRingOfIntegers( );
##  <A homalg internal ring>
##  gap> Display( ZZ );
##  Z
##  gap> m := LeftSubmodule( "2", ZZ );
##  <A principal (left) ideal given by a cyclic generator>
##  gap> Display( m );
##  [ [  2 ] ]
##  
##  A (left) ideal generated by the entry of the above matrix
##  gap> K := LeftSubmodule( "3", ZZ );
##  <A principal (left) ideal given by a cyclic generator>
##  gap> Display( K );
##  [ [  3 ] ]
##  
##  A (left) ideal generated by the entry of the above matrix
##  gap> J := Intersect( K, m^3 );
##  <A principal (left) ideal given by a cyclic generator>
##  gap> Display( J );
##  [ [  24 ] ]
##  
##  A (left) ideal generated by the entry of the above matrix
##  gap> Jm := SubmoduleQuotient( J, m );
##  <A principal (left) ideal given by a cyclic generator>
##  gap> Display( Jm );
##  [ [  -12 ] ]
##  
##  A (left) ideal generated by the entry of the above matrix
##  gap> J_m := Saturate( J, m );
##  <A principal (left) ideal given by a cyclic generator>
##  gap> Display( J_m );
##  [ [  -3 ] ]
##  
##  A (left) ideal generated by the entry of the above matrix
##  gap> J_m = K;
##  true
##  ]]></Example>
##  <#/GAPDoc>

LoadPackage( "homalg" );

ZZ := HomalgRingOfIntegers( );

m := LeftSubmodule( "2", ZZ );

K := LeftSubmodule( "3", ZZ );

J := Intersect( K, m^3 );

Jm := SubmoduleQuotient( J, m );

J_m := Saturate( J, m );
