$indexonly=0;

sub processFile($) {
	my($ffull) = @_;
	$f=substr($f,rindex($f,'/')+1);
	print "processing ".$ffull."\n";

	open(MNF,"<$ffull") or die "could not open file $ffull!\n";

	my $n='';
	while($l=<MNF>) {
		chomp($l);
		if($l=~m/<Name.*?>(.*?)<\/Name>/) {
			$n=$1;
			break;
		}
	}


	if($n eq '') {
			die "file $ffull has invalid format!\n";
	}

	$n=~s/twist/~/gi;
	$n=~s/\s+//g;

	if($n=~m/SurfOG([0-9]+)/) {
		$gint=int($1);
		if($gint==1) {
			$n="T^2";
		} else {
			$n="(T^2)#".$gint;
		}
	}

	print IDXFILE "\t<SimplicialComplex>\n\t\t<Name type=\"SCString\">$n</Name>\n\t\t<File type=\"SCString\">".$ffull."</File>\n\t</SimplicialComplex>\n";

	close(MNF);
}

sub recurse($) {
  my($path) = @_;

  ## append a trailing / if it's not there
  $path .= '/' if($path !~ /\/$/);

  ## loop through the files contained in the directory
  for my $eachFile (glob($path.'*')) {

    ## if the file is a directory
    if( -d $eachFile) {
      ## pass the directory to the routine ( recursion )
      recurse($eachFile);
    } else {
      ## print the file ... tabbed for readability
      processFile($eachFile);
    }
  }
}



#
# MAIN PROGRAM
#

#open index file
open(IDXFILE,">complexes.index") or die "could not open index file!\n";
print IDXFILE "<SimplicialComplexLibrary>\n";

# recurse("manifolds");



#loop over all transitive manifolds
for($dim=2;$dim<=11;$dim++){

	$fname=$dim."_manifolds";

	open(INFILE, "<$fname") or die "could not open $fname!\n";
	open(INFILE2, "<$fname.type") or die "could not open $fname.type!\n";
	$filename="";
	$count=0;
	$parsing=0;
	while ($line = <INFILE>) {
		$line=~s/\s*//gis;
		chomp($line);

		if(!$line) {
			if($parsing) {
				if($indexonly==0) {
					print "closing file $filename.\n";
					print OUTFILE "$facets</Facets>\n";
					print OUTFILE "</SimplicialComplex>\n";
					close(OUTFILE);
				}
				$parsing=0;
		    } else {
				die "error 1 in input while parsing dim $dim, file $filename, item $name\n";
			}
		}


		if($line=~m/(manifold_.*?)\s*=(.*)/) {
			if($parsing) {
				die "error 2 in input while parsing dim $dim, file $filename, item $name\n";
			}

			$parsing=1;
			$name=$1;
			$facets=$2;
			$count++;

			$type=<INFILE2>;
			chomp($type);
			if(not $type=~m/^$name/) { die "error in input, $name!\n"; }

			if($dim==2) {
				if($type=~m/.*?=.*/) {
					$type=~s/.*?=(.*?)\s*/$1/;
				} else {
					$type=~s/^\s*$name[:]?\s*(\(.*?\)).*$/$1/;
				}
			} else {
				$type=~s/$name:\s*//;
			}

			$type=~s/twist/~/gi;

			#$desc='';
			if($dim eq 2 and $type=~m/\(\s*([+-])\s*;\s*([0-9]+)\s*\)/) {
			   my $orientable=$1;
			   my $genus=$2;

			   #print "got surface ($orientable,$genus)\n";

			   if($orientable eq "+") {
				   if($genus==1) {
					   $type="T^2";
				   } else {
					   $type="(T^2)#$genus";
				   }
			   } else {
				   if($genus==1) {
					   $type="P^2";
				   } elsif($genus==2) {
					   $type="K^2";
				   } else {
					   $type="(P^2)#$genus";
				   }
			   }

			   #$desc="\n\t<Description type=\"SCString\">".($orientable eq "+"?"orientable":"non-orientable")." surface of genus ".$genus.($orientable eq "-"?"/2":"")". Vertex transitive automorphism group. Lutz: $name.</Description>";

			   #print("type $type\n");

			}



			$type=~s/\s+//g;
			$type=~s/Kleinbottle/K^2/gi;
			$type=~s/twist/~/gi;

			if($indexonly==0) {
				$filename="transitive/".$fname."_".$count.".sc";
				print "writing to file $filename...\n";
				open(OUTFILE, ">$filename") or die "could not open $filename!\n";;
				print OUTFILE "<SimplicialComplex type=\"SCSimplicialComplex\">\n\t<Name type=\"SCString\">$type (VT)</Name>\n\t<Reference type=\"SCString\">$name in F.H.Lutz: 'The Manifold Page', http://www.math.tu-berlin.de/diskregeom/stellar/,\nF.H.Lutz: 'Triangulated manifolds with few vertices and vertex-transitive group actions', Doctoral Thesis TU Berlin 1999, Shaker-Verlag, Aachen 1999</Reference>\n\t<Facets type=\"SCArray\">";
			}

			print IDXFILE "\t<SimplicialComplex>\n\t\t<Name type=\"SCString\">$type (vertex transitive, Lutz: $name)</Name>\n\t\t<File type=\"SCString\">$filename</File>\n\t</SimplicialComplex>\n";
		} elsif($line ne "") {
			if($parsing!=1) {
				die "error 3 in input while parsing dim $dim, file $filename, item \n, line: \"$line\"";
			}
			$facets.=$line;
		}

	}

	if($parsing) {
		if($indexonly==0) {
			print "closing file $filename.\n";
			print OUTFILE "$facets</Facets>\n";
			print OUTFILE "</SimplicialComplex>\n";
			close(OUTFILE);
		}
		$parsing=0;
	}

	close(INFILE);
	close(INFILE2);
}

#process other manifolds

print IDXFILE "</SimplicialComplexLibrary>\n\n";
close(IDXFILE);