################################################################################
##
##  simpcomp / init.g
##
##  init.g for package simpcomp
##
##  $Id: init.g 8 2009-11-26 13:05:35Z jonathan $
##
################################################################################

DeclareInfoClass("InfoSimpcomp");
SetInfoLevel(InfoSimpcomp,1);

DeclareGlobalVariable("SCIntFunc");
InstallValue(SCIntFunc,rec());
MakeReadWriteGlobal("SCIntFunc");

# checks if package 'homology' is present and working
SCIntFunc.SetupHomology:=function()
	local success;
	success:=LoadPackage("homology", ">=1.4.2", false);
	if success=true and IsBoundGlobal("SimplicialHomology") and not ForAny( ["homology_gap","smithform_gap"], file -> Filename(DirectoriesPackagePrograms("homology"), file) = fail) then
		return true;
	else
		return false;
	fi;
end;

# checks if package 'GRAPE' is present and working
SCIntFunc.SetupGrape:=function()
	local success;
	success:=LoadPackage("grape", ">=4.2", false);
	if success=true and IsBoundGlobal("AutGroupGraph") and IsBoundGlobal("EdgeOrbitsGraph") and not ForAny( ["drcanon4", "drtogap4", "gap4todr"], file -> Filename(DirectoriesPackagePrograms("grape"),file)=fail) and ForAny(["dreadnaut","dreadnautB"], file -> Filename(DirectoriesPackagePrograms("grape"),file)<>fail) then
		return true;
	else
		return false;
	fi;
end;

if(SCIntFunc.SetupHomology()=false) then
	Info(InfoSimpcomp+InfoWarning,1,"simpcomp: package `homology' not installed or its binaries are not available, falling back to (slower) internal homology algorithms.");
fi;


if(SCIntFunc.SetupGrape()=false) then
	Info(InfoSimpcomp+InfoWarning,1,"simpcomp: package `GRAPE' not installed or its binaries are not available, falling back to (slower) internal algorithms for automorphism group computation.");
fi;


DeclareGlobalVariable("SCSettings");
DeclareGlobalFunction("SCInfoLevel");

ReadPackage("simpcomp","lib/propobject.gd");
ReadPackage("simpcomp","lib/tools.gd");
ReadPackage("simpcomp","lib/complex.gd");
ReadPackage("simpcomp","lib/io.gd");
ReadPackage("simpcomp","lib/lib.gd");
ReadPackage("simpcomp","lib/glprops.gd");
ReadPackage("simpcomp","lib/operations.gd");
ReadPackage("simpcomp","lib/labelops.gd");
ReadPackage("simpcomp","lib/generate.gd");
ReadPackage("simpcomp","lib/bistellar.gd");
ReadPackage("simpcomp","lib/homology.gd");
ReadPackage("simpcomp","lib/normalsurface.gd");
