################################################################################
##
##  simpcomp / bistellar.gd
##
##  Functions for bistellar moves  
##
##  $Id: bistellar.gd 2 2009-11-17 13:04:18Z felix $
##
################################################################################

## <#GAPDoc Label="bistellar">
## <Section Label="sec:bftheory"> <Heading>Theory</Heading>
## Bistellar flips, first introduced by Pachner in <Cite Key="Pachner87KonstrMethKombHomeo"/> and sometimes also refered to as Pachner moves, can prove useful for the following tasks:
## <Enum>
## <Item> Try to decide whether two closed simplicial pseudomanifolds are PL-homeomorphic.</Item>
## <Item> Reduce a given triangulation without changing its PL-type.</Item>
## <Item> Verify whether a closed simplicial pseudomanifold is a combinatorial manifold.</Item>
## </Enum>
## Bistellar moves are a convenient means to prove the PL equivalence of triangulations, see <Cite Key="Lutz03TrigMnfFewVertVertTrans" /> for an example.<P/>
## Since bistellar flips do not respect the combinatorial properties of a complex, no attention to the original vertex labels is payed, i. e. the flipped complex will be relabeled whenever its vertex labels become different from the standard labeling (for example after every reverse 0-move).
## </Section>
##
## <Section>
## <Heading>Functions for bistellar flips</Heading>
##		
##
## <#Include Label="SCBistellarOptions"/>
##
## <#Include Label="SCEquivalent"/>
## <#Include Label="SCExamineComplexBistellar"/>
## <#Include Label="SCIntFunc.SCChooseMove"/>
## <#Include Label="SCIsKStackedSphere"/>
## <#Include Label="SCIsManifold"/>
## <#Include Label="SCIsMovableComplex"/>
## <#Include Label="SCMove"/>
## <#Include Label="SCMoves"/>
## <#Include Label="SCRMoves"/>
## <#Include Label="SCReduceAsSubcomplex"/>
## <#Include Label="SCReduceComplex"/>
## <#Include Label="SCReduceComplexEx"/>
##
## </Section>
##<#/GAPDoc>

DeclareGlobalVariable("SCBistellarOptions");
DeclareGlobalFunction("SCEquivalent");
DeclareGlobalFunction("SCExamineComplexBistellar");
DeclareGlobalFunction("SCIsKStackedSphere");
DeclareGlobalFunction("SCIsManifold");
DeclareGlobalFunction("SCIsMovableComplex");
DeclareGlobalFunction("SCMove");
DeclareGlobalFunction("SCMoves");
DeclareGlobalFunction("SCRMoves");
DeclareGlobalFunction("SCReduceAsSubcomplex");
DeclareGlobalFunction("SCReduceComplex");
DeclareGlobalFunction("SCReduceComplexEx");

################################################################################
