################################################################################
##
##  simpcomp / glprops.gd
##
##  Compute Global Properties of Simplicial Complexes.  
##
##  $Id: glprops.gd 8 2009-11-26 13:05:35Z jonathan $
##
################################################################################

## GAPDoc include
## <#GAPDoc Label="glprops">
## <Section> 
## <Heading>Computing properties of simplicial complexes</Heading>
##
## The following functions compute some basic properties of a simplicial complex. In particular, the facets of the complex are not altered by these functions.<P/>
## Note: Every simplicial complex is internally stored with the standard vertex labeling from <M>1</M> to <M>n</M> and a maptable to restore the original vertex labeling. Thus, we have to relabel some of the complex properties (facets, face lattice, generators, etc...) whenever we want to return them to the user. As a consequence, some of the functions exist twice, one of them with the appendix "Ex". These functions return the standard labeling whereas the other ones relabel the result to the original labeling.
##
## <#Include Label="SCAltshulerSteinberg"/>
## <#Include Label="SCAutomorphismGroup"/>
## <#Include Label="SCAutomorphismGroupInternal"/>
## <#Include Label="SCBoundary"/>
## <#Include Label="SCDim"/>
## <#Include Label="SCDualGraph"/>
## <#Include Label="SCEulerCharacteristic"/>
## <#Include Label="SCFVector"/>
## <#Include Label="SCFaceLattice"/>
## <#Include Label="SCFaceLatticeEx"/>
## <#Include Label="SCFaces"/>
## <#Include Label="SCFacesEx"/>
## <#Include Label="SCFacets"/>
## <#Include Label="SCFacetsEx"/>
## <#Include Label="SCFpBettiNumbers"/>
## <#Include Label="SCFundamentalGroup"/>
## <#Include Label="SCGVector"/>
## <#Include Label="SCGenerators"/>
## <#Include Label="SCGeneratorsEx"/>
## <#Include Label="SCHVector"/>
## <#Include Label="SCHasBoundary"/>
## <#Include Label="SCHasInterior"/>
## <#Include Label="SCHomology"/>
## <#Include Label="SCIncidences"/>
## <#Include Label="SCIncidencesEx"/>
## <#Include Label="SCInterior"/>
## <#Include Label="SCIsCentrallySymmetric"/>
## <#Include Label="SCIsConnected"/>
## <#Include Label="SCIsEmpty"/>
## <#Include Label="SCIsEulerianManifold"/>
## <#Include Label="SCIsHomologySphere"/>
## <#Include Label="SCIsInKd"/>
## <#Include Label="SCIsKNeighborly"/>
## <#Include Label="SCIsOrientable"/>
## <#Include Label="SCIsPseudoManifold"/>
## <#Include Label="SCIsPure"/>
## <#Include Label="SCIsShellable"/>
## <#Include Label="SCIsStronglyConnected"/>
## <#Include Label="SCMinimalNonFaces"/>
## <#Include Label="SCMinimalNonFacesEx"/>
## <#Include Label="SCName"/>
## <#Include Label="SCNeighborliness"/>
## <#Include Label="SCOrientation"/>
## <#Include Label="SCSkel"/>
## <#Include Label="SCSkelEx"/>
## <#Include Label="SCSpanningTree"/>
## <#Include Label="SCVertices"/>
## <#Include Label="SCVerticesEx"/>
## </Section>
## <#/GAPDoc>

DeclareGlobalFunction("SCAltshulerSteinberg");
DeclareGlobalFunction("SCAutomorphismGroup");
DeclareGlobalFunction("SCAutomorphismGroupInternal");
DeclareGlobalFunction("SCBoundary");
DeclareGlobalFunction("SCDim");
DeclareGlobalFunction("SCDualGraph");
DeclareGlobalFunction("SCEulerCharacteristic");
DeclareGlobalFunction("SCFVector");
DeclareGlobalFunction("SCFaceLattice");
DeclareGlobalFunction("SCFaceLatticeEx");
DeclareGlobalFunction("SCFaces");
DeclareGlobalFunction("SCFacesEx");
DeclareGlobalFunction("SCFacets");
DeclareGlobalFunction("SCFacetsEx");
DeclareGlobalFunction("SCFpBettiNumbers");
DeclareGlobalFunction("SCFundamentalGroup");
DeclareGlobalFunction("SCGVector");
DeclareGlobalFunction("SCGenerators");
DeclareGlobalFunction("SCGeneratorsEx");
DeclareGlobalFunction("SCHVector");
DeclareGlobalFunction("SCHasBoundary");
DeclareGlobalFunction("SCHasInterior");
DeclareGlobalFunction("SCHomology");
DeclareGlobalFunction("SCIncidences");
DeclareGlobalFunction("SCIncidencesEx");
DeclareGlobalFunction("SCInterior");
DeclareGlobalFunction("SCIsCentrallySymmetric");
DeclareGlobalFunction("SCIsConnected");
DeclareGlobalFunction("SCIsEmpty");
DeclareGlobalFunction("SCIsEulerianManifold");
DeclareGlobalFunction("SCIsHomologySphere");
DeclareGlobalFunction("SCIsInKd");
DeclareGlobalFunction("SCIsKNeighborly");
DeclareGlobalFunction("SCIsOrientable");
DeclareGlobalFunction("SCIsPseudoManifold");
DeclareGlobalFunction("SCIsPure");
DeclareGlobalFunction("SCIsShellable");
DeclareGlobalFunction("SCIsStronglyConnected");
#DeclareGlobalFunction("SCMinimalNonFaces");
DeclareGlobalFunction("SCMinimalNonFacesEx");
DeclareGlobalFunction("SCName");
DeclareGlobalFunction("SCNeighborliness");
DeclareGlobalFunction("SCOrientation");
DeclareGlobalFunction("SCSkel");
DeclareGlobalFunction("SCSkelEx");
DeclareGlobalFunction("SCSpanningTree");
DeclareGlobalFunction("SCVertices");
DeclareGlobalFunction("SCVerticesEx");

