################################################################################
##
##  simpcomp / homology.gd
##
##  Homology related functions
##
##  $Id: homology.gd 2 2009-11-17 13:04:18Z felix $
##
################################################################################

## <#GAPDoc Label="homology">
##
## Although <Package>simpcomp</Package> relies on the GAP package ''homology'' <Cite Key="Dumas04Homology" /> for its homology calculations due to efficiency reasons, the package contains some additional homology related functionality, which will be explained in this chapter. 
##
## <Section>
## <Heading>Homology computation</Heading>
##
## Apart from calculating boundaries of simplices, boundary matrices or the simplicial homology of a given complex, <Package>simpcomp</Package> is also able to compute a basis of the homology groups.
##
## <#Include Label="SCBoundaryOperatorMatrix"/>
## <#Include Label="SCBoundarySimplex"/>
## <#Include Label="SCHomologyBasis"/>
## <#Include Label="SCHomologyBasisAsSimplices"/>
## <#Include Label="SCHomologyInternal"/>
##
## </Section>
##
## <Section>
## <Heading>Cohomology computation</Heading>
## 
## <Package>simpcomp</Package> can also compute the cohomology groups of simplicial complexes, bases of these cohomology groups, the cup product of two cocycles and the intersection form of (orientable) 4-manifolds. Due to the enormous amount of resources needed for the computation of exact eigenvalues, the signature of the intersection form cannot be computed within <C>GAP</C>. However, in some cases the <C>GAP</C> function <C>Eigenvalues</C> can be used.
##
## <#Include Label="SCCoboundaryOperatorMatrix"/>
## <#Include Label="SCCohomology"/>
## <#Include Label="SCCohomologyBasis"/>
## <#Include Label="SCCohomologyBasisAsSimplices"/>
## <#Include Label="SCCupProduct"/>
## <#Include Label="SCIntersectionForm"/>
## <#Include Label="SCIntersectionFormParity"/>
## <#Include Label="SCIntersectionFormDimensionality"/>
## <#Include Label="SCIntersectionFormSignature"/>
##
## </Section>
##
##
##<#/GAPDoc>

DeclareGlobalFunction("SCBoundaryOperatorMatrix");
DeclareGlobalFunction("SCBoundarySimplex");
DeclareGlobalFunction("SCCoboundaryOperatorMatrix");
DeclareGlobalFunction("SCCohomology");
DeclareGlobalFunction("SCCohomologyBasis");
DeclareGlobalFunction("SCCohomologyBasisAsSimplices");
DeclareGlobalFunction("SCHomologyBasis");
DeclareGlobalFunction("SCHomologyBasisAsSimplices");
DeclareGlobalFunction("SCHomologyInternal");

DeclareGlobalFunction("SCCupProduct");
DeclareGlobalFunction("SCIntersectionForm");
DeclareGlobalFunction("SCIntersectionFormParity");
DeclareGlobalFunction("SCIntersectionFormDimensionality");
DeclareGlobalFunction("SCIntersectionFormSignature");

################################################################################

