################################################################################
##
##  simpcomp / lib.gd
##
##  simplicial complexes library
##
##  $Id: lib.gd 2 2009-11-17 13:04:18Z felix $
##
################################################################################


## GAPDoc include
## <#GAPDoc Label="lib">
## <Section>
## <Heading>Simplicial complex library</Heading>
##		
## <Package>simpcomp</Package> contains a library of simplicial complexes on few vertices, most of them (combinatorial) triangulations of manifolds. The user can load these known triangulations from the library in order to study their properties or to construct new triangulations out of the known ones. For exmaple, a user could determine the topological type of a given triangulation -- which can be quite tedious if done by hand -- by establishing a PL equivalence to a complex in the library.<P/>
##
## Among other known triangulations, the library contains all of the vertex transitive triangulations of d-manifolds, <M>d \leq 11</M> with few (<M>n\leq 13</M> and <M>n\leq 15</M> for <M>d=2,3,9,10,11</M>) vertices classified by Frank Lutz that can be found on his ``Manifold Page'' <URL>http://www.math.tu-berlin.de/diskregeom/stellar/</URL>, along with some triangulations of sphere bundles and vertex transitive triangulations of pseudomanifolds.<P/>
##
## See <Ref Var="SCLib"/> for a naming convention used for the global library of <Package>simpcomp</Package>.
##
## <#Include Label="SCIsLibRepository"/>
##
## <#Include Label="SCLib"/>
##
## <#Include Label="SCLibAdd"/>
## <#Include Label="SCLibAllComplexes"/>
## <#Include Label="SCLibDelete"/>
## <#Include Label="SCLibDetermineTopologicalType"/>
## <#Include Label="SCLibFlush"/>
## <#Include Label="SCLibInit"/>
## <#Include Label="SCLibIsLoaded"/>
## <#Include Label="SCLibSearchByAttribute"/>
## <#Include Label="SCLibSearchByName"/>
## <#Include Label="SCLibSize"/>
## <#Include Label="SCLibUpdate"/>
## <#Include Label="SCLibStatus"/>
##
## </Section>
## <#/GAPDoc>


DeclareCategory("SCIsLibRepository",SCIsPropertyObject);

DeclareGlobalVariable("SCLib");

DeclareGlobalFunction("SCLibAdd");
DeclareGlobalFunction("SCLibAllComplexes");
DeclareGlobalFunction("SCLibDelete");
DeclareGlobalFunction("SCLibDetermineTopologicalType");
DeclareGlobalFunction("SCLibFlush");
DeclareGlobalFunction("SCLibInit");
DeclareGlobalFunction("SCLibIsLoaded");
DeclareGlobalFunction("SCLibLoad");
DeclareGlobalFunction("SCLibSearchByAttribute");
DeclareGlobalFunction("SCLibSearchByName");
DeclareGlobalFunction("SCLibSize");
DeclareGlobalFunction("SCLibStatus");
DeclareGlobalFunction("SCLibUpdate");
################################################################################
