################################################################################
##
##  simpcomp / normalsurface.gd
##
##  Normal surfaces
##
##  $Id: normalsurface.gd 8 2009-11-26 13:05:35Z jonathan $
##
################################################################################

## <#GAPDoc Label="normalsurface">
##
## <Section Label="sec:NormSurf">
## <Heading>The object types <C>SCNormalSurface</C> as asubtyope of <C>SCPropertyObject</C></Heading>
##
## The <Package>simpcomp</Package> package defines a GAP object type <C>SCNormalSurface</C> to describe normal surfaces / slicings (level sets of discrete Morse functions) in combinatorial <M>3</M>-manifolds.
## Internally <C>SCNormalSurface</C> is a subtype of <C>SCPropertyObject</C> and, thus, mostly behaves like a <C>SCSimplicialComplex</C> object (see section <Ref Chap="sec:SCSimplComplObj" />).
##
## For a very short introduction to normal surfaces see <Ref Chap="sec:NormSurfTheory"/>, for some fundamental methods and functions for <C>SCNormalSurface</C> see below.
##
## </Section>
## <Section>
## <Heading>Overloaded operators of <C>SCNormalSurface</C></Heading>
## 
## As with the object type <C>SCSimplcicialComplex</C>, <Package>simpcomp</Package> overloads some standard operations for the object type <C>SCNormalSurface</C>. See a list of overloaded operators below. 
#
##	
## <#Include Label="NSOpPlusSCInt"/>
## <#Include Label="NSOpMinusSCInt"/>
## <#Include Label="NSOpModSCInt"/>
## <#Include Label="NSOpEqSCSC"/>
## <#Include Label="NSOpUnionSCSC"/>
##	
## </Section>
##
## <Section>
## <Heading><C>SCNormalSurface</C> as a subtype of <C>Set</C></Heading>
##	
## Like objects of type <C>SCSimplicialComplex</C>, an object of type <C>SCNormalSurface</C> behaves like a GAP <C>Set</C> type. The elements of the set are given by the facets of the normal surface, grouped by their dimensionality and type, i.e. if <C>complex</C> is an object of type <C>SCNormalSurface</C>, <C>c[1]</C> refers to the 0-faces of <C>complex</C>, <C>c[2]</C> to the 1-faces, <C>c[3]</C> to the triangles and <C>c[4]</C> to the quadrilaterals.
##
## See section <Ref Chap="sec:SubtypeOfSet"/> for details.
## </Section>
## <#/GAPDoc>
##
## <#GAPDoc Label="normalsurfaceFunc">
##
## <Section>
## <Heading>Creating an <C>SCNormalSurface</C> object</Heading>
##
##  This section contains functions to construct normal surfaces from facet lists or combinatorial <M>3</M>-manifolds.
##
## <#Include Label="SCNSEmpty"/>
## <#Include Label="SCNSFromFacets"/>
## <#Include Label="SCNS"/>
## <#Include Label="SCNSSlicing"/>
##
## </Section>
##
## <Section>
## <Heading>Generating new objects from normal surfaces</Heading>
##
## <Package>simpcomp</Package> provides the possibility to copy and / or triangulate normal surfaces. Note, that other constructions like the connected sum or the cartesian product do not make sense for (embedded) normal surfaces in general.
##
## <#Include Label="SCNSCopy"/>
## <#Include Label="SCNSSubdivision"/>
##
## </Section>
##
## <Section>
## <Heading>Properties of <C>SCNormalSurface</C> objects</Heading>
##
## Although some properties of a normal surface can be computed by using the functions for simplicial complexes, there is a variety of properties needing specially designed functions. See below for a list.
##
## <#Include Label="SCNSDim"/>
## <#Include Label="SCNSEulerCharacteristic"/>
## <#Include Label="SCNSFVector"/>
## <#Include Label="SCNSFaceLattice"/>
## <#Include Label="SCNSFaceLatticeEx"/>
## <#Include Label="SCNSFpBettiNumbers"/>
## <#Include Label="SCNSGenus"/>
## <#Include Label="SCNSHomology"/>
## <#Include Label="SCNSSkel"/>
## <#Include Label="SCNSSkelEx"/>
## <#Include Label="SCNSTopologicalType"/>
##
## </Section>
## <#/GAPDoc>

DeclareCategory("SCIsNormalSurface",SCIsPropertyObject and IsSet);
DeclareGlobalFunction("SCNSFromFacets");
DeclareGlobalFunction("SCNS");
DeclareGlobalFunction("SCNSCopy");
DeclareGlobalFunction("SCNSEmpty");
DeclareGlobalFunction("SCNSNew");
DeclareOperation("SCNSNewWithProperties",[IsRecord]);
DeclareGlobalVariable("SCNSPropertyHandlers");
DeclareGlobalVariable("SCNSViewProperties");
DeclareGlobalFunction("SCNSDim");
DeclareGlobalFunction("SCNSFVector");
DeclareGlobalFunction("SCNSEulerCharacteristic");
DeclareGlobalFunction("SCNSGenus");
DeclareGlobalFunction("SCNSFaceLattice");
DeclareGlobalFunction("SCNSFaceLatticeEx");
DeclareGlobalFunction("SCNSSkel");
DeclareGlobalFunction("SCNSSkelEx");
DeclareGlobalFunction("SCNSSubdivision");
DeclareGlobalFunction("SCNSHomology");
DeclareGlobalFunction("SCNSFpBettiNumbers");
DeclareGlobalFunction("SCNSTopologicalType");
DeclareGlobalFunction("SCNSSlicing");
# DeclareGlobalFunction("SCNSAutomorphismGroup");
# DeclareGlobalFunction("SCNSIsCentrallySymmetric");
# DeclareGlobalFunction("SCNSGenerators");
# DeclareGlobalFunction("SCNSGeneratorsEx");
#DeclareGlobalFunction("SCForceRecalc");
#face lattice operations
#DeclareGlobalFunction("SCDifference");
#DeclareGlobalFunction("SCIntersection");
#DeclareGlobalFunction("SCUnion");
#label operations

DeclareOperation("+",[SCIsNormalSurface,IsInt]);
DeclareOperation("-",[SCIsNormalSurface,IsInt]);
DeclareOperation("*",[SCIsNormalSurface,IsInt]);
#DeclareOperation("mod",[SCIsNormalSurface,IsInt]);
#DeclareOperation("=",[SCIsNormalSurface,SCIsNormalSurface]);
