################################################################################
##
##  simpcomp / pkggrape.gi
##
##  Loaded when `GRAPE' package is available
##
##  $Id: pkggrape.gi 2 2009-11-17 13:04:18Z felix $
##
################################################################################

InstallGlobalFunction(SCAutomorphismGroup,
	function(complex)

	local i,j,k,idx, dg, edges, f, gamma, Gprime, G, pm, sc, structure,g,bd,
		stars,imstars,facets,pfacets,iso,allisos,verts,gens;
	
	if(not SCIsSimplicialComplex(complex)) then
		SCError()("SCAutomorphismGroup: first argument must be of type SCSimplicialComplex.\n");
		return fail;
	fi;
	
	G:=SCPropertyByName(complex,"AutomorphismGroup");
	if G<>fail and not SCIntFunc.ForceRecalcEnabled(complex) then
		return G;
	fi;
	
	
	if not IsBoundGlobal("AutGroupGraph") or not IsBoundGlobal("EdgeOrbitsGraph") then
		return SCAutomorphismGroupInternal(complex);
	fi;
	
	
	pm:=SCIsPseudoManifold(complex);
	if(pm=fail) then
		return fail;
	elif(pm=false) then
		Info(InfoSimpcomp,1,"SCAutomorphismGroup: can only compute automorphism group for pseudomanifolds.");
		return fail;
	fi;

	f:=SCFVector(complex);
	if f = fail then
		return fail;
	fi;

	#identify simplices
	if f[1] = f[Size(f)] and pm = true and SCDim(complex) = f[1]-2 then
		G:=SymmetricGroup(f[1]);
		SCPropertySet(complex,"AutomorphismGroup",G);
		SCPropertySet(complex,"AutomorphismGroupSize",Size(G));
		SCPropertySet(complex,"AutomorphismGroupTransitivity",Transitivity(G));
		structure:=StructureDescription(G);
		SCPropertySet(complex,"AutomorphismGroupStructure",structure);
		SetName(G,structure);
		return G;
	fi;
	
	dg:=SCDualGraph(complex);
	if dg = fail then
		return fail;
	fi;

	edges:=ShallowCopy(SCFacets(dg));
	if edges = fail then
		return fail;
	fi;

	verts:=SCVertices(dg);
	if verts = fail then
		return fail;
	fi;
	
	for i in [1..Size(edges)] do
		Add(edges,Reversed(edges[i]));
	od;
	
	gamma:=EdgeOrbitsGraph(Group([()]),edges,Size(verts));
	Gprime:=AutGroupGraph(gamma);
	
	if(Gprime=fail) then
		SCError()("SCAutomorphismGroup: error calculating automorphism group via GRAPE.\n");
		return fail;
	fi;
	
	SCPropertySet(dg,"AutomorphismGroup",Gprime);
	SCPropertySet(dg,"AutomorphismGroupSize",Size(Gprime));
	SCPropertySet(dg,"AutomorphismGroupTransitivity",Transitivity(Gprime));
	SCPropertySet(dg,"AutomorphismGroupStructure",StructureDescription(Gprime));
	SCPropertySet(complex,"DualGraph",dg);
	
	facets:=SCFacetsEx(complex);
	stars:=[];
	for i in [1..f[1]] do
		stars[i]:=[];
	od;
	for j in [1..Size(facets)] do
			for k in facets[j] do
				#idx[k][j]:=1;
				Add(stars[k],j);
			od;
	od;
	
			
	# for all automorphisms of dual graph
	G:=Group(());
	allisos:=[];
	gens:=[];
	for g in Gprime do
		if(g in G) then
			continue;
		fi;
		
		imstars:=List(stars,x->Set(List(x,y->y^g)));
		
		iso:=[];
		for j in [1..Size(stars)] do
			i:=Position(stars,imstars[j]);
			if i <> fail then
				iso[j]:=[j,i];
			else
				iso:=fail;
				break;
			fi;
		od;
		
		if iso <> fail then
			Add(gens,g);
			G:=Group(gens);
			Add(allisos,iso);
		fi;
	od;

	if(allisos<>[] and allisos<>[[]]) then	
		allisos:=Set(allisos);
		G:=Group(List(List(allisos,SCIntFunc.PairToList),PermList));
	else
		G:=Group(());
	fi;
	
	SCPropertySet(complex,"AutomorphismGroup",G);
	SCPropertySet(complex,"AutomorphismGroupSize",Size(G));
	SCPropertySet(complex,"AutomorphismGroupTransitivity",Transitivity(G));
	structure:=StructureDescription(G);
	SCPropertySet(complex,"AutomorphismGroupStructure",structure);
	SetName(G,structure);
	
	return G;
end);

