################################################################################
##
##  simpcomp / propobject.gd
##
##  provides object type SCPropertyObject
##
##  $Id: propobject.gi 2 2009-11-17 13:04:18Z felix $
##
################################################################################



#representation
DeclareRepresentation("SCIsPropertyObjectRep",IsComponentObjectRep,
[
	"Properties",
	"PropertiesTmp",
	"PropertyHandlers"
]);

SCPropertyObjectFamily:=NewFamily("SCPropertyObjectFamily",SCIsPropertyObject and SCIsPropertyObjectRep and IsMutable and IsCopyable);
SCPropertyObjectType:=NewType(SCPropertyObjectFamily,SCIsPropertyObject and SCIsPropertyObjectRep);


#set property handler functions
InstallMethod(SCPropertyHandlersSet,"for SCPropertyObject",
[SCIsPropertyObject,IsRecord],
function(po,handlers)
	po!.PropertyHandlers:=handlers;
	return true;
end);


#get known properties record
InstallMethod(SCProperties,"for SCPropertyObject",
[SCIsPropertyObject],
function(po)
	return po!.Properties;
end);

#return names of known properties
InstallMethod(SCPropertiesNames,"for SCPropertyObject",
[SCIsPropertyObject],
function(po)
	return RecNames(po!.Properties);
end);


# get the value of a property, fail if property does not exist
InstallMethod(SCPropertyByName,"for SCPropertyObject",
[SCIsPropertyObject,IsString],
function(po,name)
	if not IsBound(po!.Properties.(name)) then
		return fail;
	else
		return po!.Properties.(name);
	fi;
end);


#set the value of a property
InstallMethod(SCPropertySet,"for SCPropertyObject",
[SCIsPropertyObject,IsString,IsObject],
function(po,name,data)
	MakeImmutable(data);
	po!.Properties.(name):=data;
	return true;
end);

InstallMethod(SCPropertySetMutable,"for SCPropertyObject",
[SCIsPropertyObject,IsString,IsObject],
function(po,name,data)
	po!.Properties.(name):=data;
	return true;
end);



#unbind a known property
InstallMethod(SCPropertyDrop,"for SCPropertyObject",
[SCIsPropertyObject,IsString],
function(po,name)
	if not IsBound(po!.Properties.(name)) then
		return fail;
	else
		Unbind(po!.Properties.(name));
	fi;
	return true;
end);



#get known properties record
InstallMethod(SCPropertiesTmp,"for SCPropertyObject",
[SCIsPropertyObject],
function(po)
	return po!.PropertiesTmp;
end);

#return names of known properties
InstallMethod(SCPropertiesTmpNames,"for SCPropertyObject",
[SCIsPropertyObject],
function(po)
	return RecNames(po!.PropertiesTmp);
end);


# get the value of a property, fail if property does not exist
InstallMethod(SCPropertyTmpByName,"for SCPropertyObject",
[SCIsPropertyObject,IsString],
function(po,name)
	if not IsBound(po!.PropertiesTmp.(name)) then
		return fail;
	else
		return po!.PropertiesTmp.(name);
	fi;
	return true;
end);


#set the value of a property
InstallMethod(SCPropertyTmpSet,"for SCPropertyObject",
[SCIsPropertyObject,IsString,IsObject],
function(po,name,data)
	po!.PropertiesTmp.(name):=data;
	return true;
end);


#unbind a known property
InstallMethod(SCPropertyTmpDrop,"for SCPropertyObject",
[SCIsPropertyObject,IsString],
function(po,name)
	if not IsBound(po!.PropertiesTmp.(name)) then
		return fail;
	else
		Unbind(po!.PropertiesTmp.(name));
	fi;
	return true;
end);


#unbind all known properties
InstallMethod(SCFlush,
[SCIsPropertyObject],
function(po)
	po!.Properties:=rec();
	po!.PropertiesTmp:=rec();
	return true;
end);


#list all known properties
InstallMethod(SCPropertiesManaged,
[SCIsPropertyObject],
function(po)
	if(not IsBound(po!.PropertyHandlers)) then
		return fail;
	fi;
	
	return Concatenation(RecNames(po!.PropertyHandlers),RecNames(po!.Properties));
end);

InstallMethod(
	IsBound\.,"for SCPropertyObject",
	[SCIsPropertyObject, IsPosInt],
function(po, key)
	local strid;
	strid:=NameRNam(key);
	
	if(IsBound(po!.PropertyHandlers.(strid)) or IsBound(po!.Properties.(strid))) then
		return true;
	else
		return false;
	fi;
end);


#access a property
InstallMethod(
	\.,"for SCPropertyObject",
	[SCIsPropertyObject, IsPosInt],
function(po, key)
	local strid;
	strid:=NameRNam(key);
	if(IsBound(po!.PropertyHandlers.(strid))) then
		if(NumberArgumentsFunction(po!.PropertyHandlers.(strid))=1) then
			return po!.PropertyHandlers.(strid)(po);
		else
			return SCIntFunc.WrapperFunc(po!.PropertyHandlers.(strid),po);
		fi;
	elif IsBound(po!.Properties.(strid)) then
		return po!.Properties.(strid);
	else
		Info(InfoSimpcomp,1,"SCPropertyObject: unhandled property '",strid,"'. Handled properties are ",Concatenation(RecNames(po!.PropertyHandlers),RecNames(po!.Properties)),".");
		return fail;
	fi;
end);

#wrapper function for pseudo-object-orientation
SCIntFunc.WrapperFunc:=
function(func,obj)
	return function(arg) return CallFuncList(func,Concatenation([obj],arg)); end;
end;

SCIntFunc.ParseParameters:=
function(params)
	local sc,larg;

	if(SCIntFunc._SC<>fail) then
		sc:=SCIntFunc._SC;
		SCIntFunc._SC:=fail;
		larg:=params;
	else
		if(Length(params)=0 or not SCIsSimplicialComplex(params[1])) then
			return fail;
		else
			sc:=params[1];
		fi;
		larg:=params{[2..Length(params)]};
	fi;
	
	return [sc,larg];
end;
	
SCIntFunc.EnsureParameters:=
function(func,params,filters,types)
	local i;
	
	if(IsList(params) and not IsStringRep(params)) then
		if(Length(params)<>Length(filters)) then
			Info(InfoSimpcomp,1,"SCIntFunc.EnsureParameters: number of parameters and number of types do not match."); 
			return false;
		fi;
		
		for i in [1..Length(params)] do
			if(not filters[i](params[i])) then
				Info(InfoSimpcomp,1,func,": wrong arguments, expected ",types,".");
				return false;
			fi;
		od;
		return true;
	else
		return types(params);
	fi;
end;


################################################################################

